function [pvalue,SMI0,SMI]=LSIT(x,y,y_type,T)
%
% Least-Squares Independence Test
%
% Test the null hypothesis that x and y are independent.
%
% Usage:
%       [pvalue,SMI0,SMI]=LSIT(x,y,y_type,T)
%
% Input:
%    x          : dx by n input sample matrix
%    y          : dy by n output sample matrix
%    y_type     : if y_type=1, delta kernel is used for y; 
%                 otherwise (or empty) Gaussian kernel is used.
%    T          : Number of repetitions for permutation test (default: T=100)
%
% Output:
%    pvalue : pvalue
%    SMI0   : Estimate of squared-loss mutual information for original data (x,y)
%    SMI    : Estimates of squared-loss mutual information for
%             randomly permuted data (x,y_permuted)
%
% (c) Masashi Sugiyama, Department of Compter Science, Tokyo Institute of Technology, Japan.
%     sugi@cs.titech.ac.jp,

  if nargin<4
    T=100; % Number of repetitions for permutation test
  end

  n =size(x,2);
  ny=size(y,2);
  if n ~= ny
    error('x and y must have the same number of samples!!!')
  end;

  c=dist2(x,x);
  width_list=sqrt(median(c(:)))*[0.6 0.8 1 1.2 1.4];
  lambda_list=10.^[-3 -2 -1 0 1];
disp('Computing squared-loss mutual information for original data')
  SMI0=LSMI(x,y,y_type,width_list,lambda_list,n,5);
  for t=1:T %permutation test
disp(sprintf('  Computing squared-loss mutual information for permuted data (%g/%g)',t,T))
    rand('state',t);
    randn('state',t);
    y_permuted=y(:,randperm(n));
    SMI(t)=LSMI(x,y_permuted,y_type,width_list,lambda_list,n,5);
  end

  pvalue=mean(SMI>SMI0);
